@@ -134,6 +134,8 @@ class ws_connection_t final
 		using logger_t = typename Traits::logger_t;
 		using strand_t = typename Traits::strand_t;
 		using stream_socket_t = typename Traits::stream_socket_t;
+		using lifetime_monitor_t =
+			typename connection_count_limit_types<Traits>::lifetime_monitor_t;
 
 		using ws_weak_handle_t = std::weak_ptr< ws_t >;
 
@@ -144,12 +146,14 @@ class ws_connection_t final
 			//! \{
 			restinio::impl::connection_settings_handle_t< Traits > settings,
 			stream_socket_t socket,
+			lifetime_monitor_t lifetime_monitor,
 			//! \}
 			message_handler_t msg_handler )
 			:	ws_connection_base_t{ conn_id }
 			,	executor_wrapper_base_t{ socket.get_executor() }
 			,	m_settings{ std::move( settings ) }
 			,	m_socket{ std::move( socket ) }
+			,	m_lifetime_monitor{ std::move( lifetime_monitor ) }
 			,	m_timer_guard{ m_settings->create_timer_guard() }
 			,	m_input{ websocket_header_max_size() }
 			,	m_msg_handler{ std::move( msg_handler ) }
@@ -158,16 +162,18 @@ class ws_connection_t final
 			// Notify of a new connection instance.
 			m_logger.trace( [&]{
 					return fmt::format(
-						"[connection:{}] move socket to [ws_connection:{}]",
+						RESTINIO_FMT_FORMAT_STRING(
+							"[connection:{}] move socket to [ws_connection:{}]" ),
 						connection_id(),
 						connection_id() );
 			} );
 
 			m_logger.trace( [&]{
 					return fmt::format(
-						"[ws_connection:{}] start connection with {}",
+						RESTINIO_FMT_FORMAT_STRING(
+							"[ws_connection:{}] start connection with {}" ),
 						connection_id(),
-						m_socket.remote_endpoint() );
+						fmtlib_tools::streamed( m_socket.remote_endpoint() ) );
 			} );
 
 			// Inform state listener if it used.
@@ -192,7 +198,8 @@ class ws_connection_t final
 				// Notify of a new connection instance.
 				m_logger.trace( [&]{
 					return fmt::format(
-						"[ws_connection:{}] destructor called",
+						RESTINIO_FMT_FORMAT_STRING(
+							"[ws_connection:{}] destructor called" ),
 						connection_id() );
 				} );
 			}
@@ -216,7 +223,8 @@ class ws_connection_t final
 						restinio::utils::log_trace_noexcept( m_logger,
 							[&]{
 								return fmt::format(
-									"[ws_connection:{}] shutdown",
+									RESTINIO_FMT_FORMAT_STRING(
+										"[ws_connection:{}] shutdown" ),
 									connection_id() );
 							} );
 
@@ -228,7 +236,8 @@ class ws_connection_t final
 						restinio::utils::log_error_noexcept( m_logger,
 							[&]{
 								return fmt::format(
-									"[ws_connection:{}] shutdown operation error: {}",
+									RESTINIO_FMT_FORMAT_STRING(
+										"[ws_connection:{}] shutdown operation error: {}" ),
 									connection_id(),
 									ex.what() );
 							} );
@@ -253,7 +262,8 @@ class ws_connection_t final
 						restinio::utils::log_trace_noexcept( m_logger,
 							[&]{
 								return fmt::format(
-									"[ws_connection:{}] kill",
+									RESTINIO_FMT_FORMAT_STRING(
+										"[ws_connection:{}] kill" ),
 									connection_id() );
 							} );
 
@@ -267,7 +277,8 @@ class ws_connection_t final
 						restinio::utils::log_error_noexcept( m_logger,
 							[&]{
 								return fmt::format(
-									"[ws_connection:{}] kill operation error: {}",
+									RESTINIO_FMT_FORMAT_STRING(
+										"[ws_connection:{}] kill operation error: {}" ),
 									connection_id(),
 									ex.what() );
 							} );
@@ -304,7 +315,8 @@ class ws_connection_t final
 							status_code_t::unexpected_condition,
 							[&]{
 								return fmt::format(
-									"[ws_connection:{}] unable to init read: {}",
+									RESTINIO_FMT_FORMAT_STRING(
+										"[ws_connection:{}] unable to init read: {}" ),
 									connection_id(),
 									ex.what() );
 							} );
@@ -338,8 +350,9 @@ class ws_connection_t final
 						{
 							m_logger.warn( [&]{
 								return fmt::format(
-										"[ws_connection:{}] cannot write to websocket: "
-										"write operations disabled",
+										RESTINIO_FMT_FORMAT_STRING(
+											"[ws_connection:{}] cannot write to websocket: "
+											"write operations disabled" ),
 										connection_id() );
 							} );
 						}
@@ -350,7 +363,8 @@ class ws_connection_t final
 							status_code_t::unexpected_condition,
 							[&]{
 								return fmt::format(
-									"[ws_connection:{}] unable to write data: {}",
+									RESTINIO_FMT_FORMAT_STRING(
+										"[ws_connection:{}] unable to write data: {}" ),
 									connection_id(),
 									ex.what() );
 							} );
@@ -371,7 +385,8 @@ class ws_connection_t final
 					restinio::utils::log_trace_noexcept( m_logger,
 						[&]{
 							return fmt::format(
-									"[ws_connection:{}] close socket",
+									RESTINIO_FMT_FORMAT_STRING(
+										"[ws_connection:{}] close socket" ),
 									connection_id() );
 						} );
 
@@ -482,7 +497,8 @@ class ws_connection_t final
 		{
 			m_logger.trace( [&]{
 				return fmt::format(
-						"[ws_connection:{}] start reading header",
+						RESTINIO_FMT_FORMAT_STRING(
+							"[ws_connection:{}] start reading header" ),
 						connection_id() );
 			} );
 
@@ -507,7 +523,8 @@ class ws_connection_t final
 		{
 			m_logger.trace( [&]{
 				return fmt::format(
-						"[ws_connection:{}] continue reading message",
+						RESTINIO_FMT_FORMAT_STRING(
+							"[ws_connection:{}] continue reading message" ),
 						connection_id() );
 			} );
 
@@ -529,7 +546,9 @@ class ws_connection_t final
 								status_code_t::unexpected_condition,
 								[&]{
 									return fmt::format(
-										"[ws_connection:{}] after read header callback error: {}",
+										RESTINIO_FMT_FORMAT_STRING(
+											"[ws_connection:{}] after read header "
+											"callback error: {}" ),
 										connection_id(),
 										ex.what() );
 								} );
@@ -546,7 +565,7 @@ class ws_connection_t final
 				status_code_t::connection_lost,
 				[&]{
 					return fmt::format(
-						"[ws_connection:{}] {}: {}",
+						RESTINIO_FMT_FORMAT_STRING( "[ws_connection:{}] {}: {}" ),
 						connection_id(),
 						desc,
 						ec.message() );
@@ -563,7 +582,8 @@ class ws_connection_t final
 			{
 				m_logger.trace( [&]{
 					return fmt::format(
-							"[ws_connection:{}] received {} bytes",
+							RESTINIO_FMT_FORMAT_STRING(
+								"[ws_connection:{}] received {} bytes" ),
 							this->connection_id(),
 							length );
 				} );
@@ -602,7 +622,8 @@ class ws_connection_t final
 		{
 			m_logger.trace( [&]{
 				return fmt::format(
-						"[ws_connection:{}] start handling {} ({:#x})",
+						RESTINIO_FMT_FORMAT_STRING(
+							"[ws_connection:{}] start handling {} ({:#x})" ),
 						connection_id(),
 						opcode_to_string( md.m_opcode ),
 						static_cast<std::uint16_t>(md.m_opcode) );
@@ -615,7 +636,8 @@ class ws_connection_t final
 			{
 				m_logger.error( [&]{
 					return fmt::format(
-							"[ws_connection:{}] invalid header",
+							RESTINIO_FMT_FORMAT_STRING(
+								"[ws_connection:{}] invalid header" ),
 							connection_id() );
 				} );
 
@@ -731,7 +753,9 @@ class ws_connection_t final
 									status_code_t::unexpected_condition,
 									[&]{
 										return fmt::format(
-											"[ws_connection:{}] after read payload callback error: {}",
+											RESTINIO_FMT_FORMAT_STRING(
+												"[ws_connection:{}] after read payload "
+												"callback error: {}" ),
 											connection_id(),
 											ex.what() );
 									} );
@@ -752,7 +776,8 @@ class ws_connection_t final
 			{
 				m_logger.trace( [&]{
 					return fmt::format(
-							"[ws_connection:{}] received {} bytes",
+							RESTINIO_FMT_FORMAT_STRING(
+								"[ws_connection:{}] received {} bytes" ),
 							this->connection_id(),
 							length );
 				} );
@@ -823,7 +848,8 @@ class ws_connection_t final
 				{
 					m_logger.error( [&]{
 						return fmt::format(
-								"[ws_connection:{}] execute handler error: {}",
+								RESTINIO_FMT_FORMAT_STRING(
+									"[ws_connection:{}] execute handler error: {}" ),
 								connection_id(),
 								ex.what() );
 					} );
@@ -880,7 +906,8 @@ class ws_connection_t final
 		{
 			m_logger.error( [&]{
 					return fmt::format(
-							"[ws_connection:{}] invalid paload",
+							RESTINIO_FMT_FORMAT_STRING(
+								"[ws_connection:{}] invalid paload" ),
 							connection_id() );
 				} );
 
@@ -941,7 +968,9 @@ class ws_connection_t final
 					{
 						m_logger.trace( [&]{
 							return fmt::format(
-									"[ws_connection:{}] got close frame from peer, status: {}",
+									RESTINIO_FMT_FORMAT_STRING(
+										"[ws_connection:{}] got close frame from "
+										"peer, status: {}" ),
 									connection_id(),
 									static_cast<std::uint16_t>(
 											status_code_from_bin( m_input.m_payload )) );
@@ -978,7 +1007,8 @@ class ws_connection_t final
 
 						m_logger.trace( [&]{
 							return fmt::format(
-									"[ws_connection:{}] expected close frame came",
+									RESTINIO_FMT_FORMAT_STRING(
+										"[ws_connection:{}] expected close frame came" ),
 									connection_id() );
 						} );
 					}
@@ -1018,7 +1048,8 @@ class ws_connection_t final
 				{
 					m_logger.trace( [&]{
 						return fmt::format(
-								"[ws_connection:{}] user sends close frame",
+								RESTINIO_FMT_FORMAT_STRING(
+									"[ws_connection:{}] user sends close frame" ),
 								connection_id() );
 					} );
 
@@ -1040,7 +1071,9 @@ class ws_connection_t final
 			{
 				m_logger.warn( [&]{
 					return fmt::format(
-							"[ws_connection:{}] try to write while socket is closed",
+							RESTINIO_FMT_FORMAT_STRING(
+								"[ws_connection:{}] try to write while "
+								"socket is closed" ),
 							connection_id() );
 				} );
 
@@ -1078,8 +1111,9 @@ class ws_connection_t final
 			{
 				m_logger.trace( [&]{
 					return fmt::format(
-						"[ws_connection:{}] start next write group, "
-						"size: {}",
+						RESTINIO_FMT_FORMAT_STRING(
+							"[ws_connection:{}] start next write group, "
+							"size: {}" ),
 						this->connection_id(),
 						next_write_group->items_count() );
 				} );
@@ -1125,7 +1159,8 @@ class ws_connection_t final
 					status_code_t::unexpected_condition,
 					[&]{
 						return fmt::format(
-							"[ws_connection:{}] handle_current_write_ctx failed: {}",
+							RESTINIO_FMT_FORMAT_STRING(
+								"[ws_connection:{}] handle_current_write_ctx failed: {}" ),
 							connection_id(),
 							ex.what() );
 					} );
@@ -1140,9 +1175,10 @@ class ws_connection_t final
 
 			m_logger.trace( [&]{
 				return fmt::format(
-					"[ws_connection:{}] sending data with "
-					"buf count: {}, "
-					"total size: {}",
+					RESTINIO_FMT_FORMAT_STRING(
+						"[ws_connection:{}] sending data with "
+						"buf count: {}, "
+						"total size: {}" ),
 					connection_id(),
 					bufs.size(),
 					op.size() ); } );
@@ -1166,7 +1202,9 @@ class ws_connection_t final
 							{
 								m_logger.trace( [&]{
 									return fmt::format(
-											"[ws_connection:{}] outgoing data was sent: {} bytes",
+											RESTINIO_FMT_FORMAT_STRING(
+												"[ws_connection:{}] outgoing data was "
+												"sent: {} bytes" ),
 											connection_id(),
 											written );
 								} );
@@ -1180,7 +1218,9 @@ class ws_connection_t final
 								status_code_t::unexpected_condition,
 								[&]{
 									return fmt::format(
-										"[ws_connection:{}] after write callback error: {}",
+										RESTINIO_FMT_FORMAT_STRING(
+											"[ws_connection:{}] after write "
+											"callback error: {}" ),
 										connection_id(),
 										ex.what() );
 								} );
@@ -1195,7 +1235,8 @@ class ws_connection_t final
 			// Finishing writing this group.
 			m_logger.trace( [&]{
 				return fmt::format(
-						"[ws_connection:{}] finishing current write group",
+						RESTINIO_FMT_FORMAT_STRING(
+							"[ws_connection:{}] finishing current write group" ),
 						this->connection_id() );
 			} );
 
@@ -1221,7 +1262,8 @@ class ws_connection_t final
 					status_code_t::connection_lost,
 					[&]{
 						return fmt::format(
-							"[ws_connection:{}] unable to write: {}",
+							RESTINIO_FMT_FORMAT_STRING(
+								"[ws_connection:{}] unable to write: {}" ),
 							connection_id(),
 							ec.message() );
 					} );
@@ -1234,7 +1276,8 @@ class ws_connection_t final
 				{
 					m_logger.error( [&]{
 						return fmt::format(
-							"[ws_connection:{}] notificator error: {}",
+							RESTINIO_FMT_FORMAT_STRING(
+								"[ws_connection:{}] notificator error: {}" ),
 							connection_id(),
 							ex.what() );
 					} );
@@ -1248,6 +1291,13 @@ class ws_connection_t final
 		//! Connection.
 		stream_socket_t m_socket;
 
+		/*!
+		 * @brief Monitor of the connection lifetime.
+		 *
+		 * @since v.0.6.12
+		 */
+		lifetime_monitor_t m_lifetime_monitor;
+
 		//! Timers.
 		//! \{
 		static ws_connection_t &
@@ -1277,8 +1327,10 @@ class ws_connection_t final
 						conn_object.trigger_error_and_close(
 							status_code_t::unexpected_condition,
 							[&] {
-								return fmt::format( "[connection: {}] unexpected "
-										"error during timeout handling: {}",
+								return fmt::format(
+										RESTINIO_FMT_FORMAT_STRING(
+											"[connection: {}] unexpected "
+											"error during timeout handling: {}" ),
 										conn_object.connection_id(),
 										x.what() );
 							} );
@@ -1300,7 +1352,8 @@ class ws_connection_t final
 			{
 				m_logger.trace( [&]{
 					return fmt::format(
-							"[wd_connection:{}] write operation timed out",
+							RESTINIO_FMT_FORMAT_STRING(
+								"[wd_connection:{}] write operation timed out" ),
 							connection_id() );
 					} );
 				m_close_frame_to_peer.disable();
@@ -1311,7 +1364,9 @@ class ws_connection_t final
 			{
 				m_logger.trace( [&]{
 					return fmt::format(
-							"[wd_connection:{}] waiting for close-frame from peer timed out",
+							RESTINIO_FMT_FORMAT_STRING(
+								"[wd_connection:{}] waiting for close-frame "
+								"from peer timed out" ),
 							connection_id() );
 					} );
 				close_impl();
